@@ -3,7 +3,7 @@
  *
  * Extended Tcl pipe, copyfile and lgets commands.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1993 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1995 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXfilecmds.c,v 3.1 1994/01/23 17:00:28 markd Exp $
+ * $Id: tclXfilecmds.c,v 5.0 1995/07/25 05:42:27 markd Rel $
  *-----------------------------------------------------------------------------
  */
 /* 
@@ -20,7 +20,7 @@
  * Note: List parsing code stolen from Tcl distribution file tclUtil.c,
  * procedure TclFindElement.
  *-----------------------------------------------------------------------------
- * Copyright (c) 1987-1993 The Regents of the University of California.
+ * Copyright (c) 1987-1994 The Regents of the University of California.
  * All rights reserved.
  *
  * Permission is hereby granted, without written agreement and without
@@ -44,6 +44,10 @@
 
 #include "tclExtdInt.h"
 
+static char *FILE_ID_OPT = "-fileid";
+static char *FILE_ID_NOT_AVAIL =
+    "the -fileid option is not available on this system";
+
 /*
  * Prototypes of internal functions.
  */
@@ -58,6 +62,16 @@ GetsListElement _ANSI_ARGS_((Tcl_Interp    *interp,
                              FILE          *filePtr,
                              Tcl_DString   *bufferPtr,
                              int           *idxPtr));
+
+static int
+TruncateByPath  _ANSI_ARGS_((Tcl_Interp  *interp,
+                             char        *filePath,
+                             off_t        newSize));
+
+static int
+TruncateByHandle  _ANSI_ARGS_((Tcl_Interp  *interp,
+                               char        *fileHandle,
+                               off_t        newSize));
 
 /*
  *-----------------------------------------------------------------------------
@@ -122,7 +136,9 @@ Tcl_PipeCmd (clientData, interp, argc, argv)
  *
  * CopyOpenFile --
  * 
- *  Utility function to copy an open file to another open file.
+ *  Utility function to copy an open file to another open file.  Handles
+ * non-blocking I/O in the same manner as gets.  It doesn't return an
+ * error when EWOULDBLOCK or EAGAIN is returned if some data has been read.
  *
  * Parameters:
  *   o interp (I) - Error messages are returned in the interpreter.
@@ -131,7 +147,6 @@ Tcl_PipeCmd (clientData, interp, argc, argv)
  *   o outFilePtr (I) - Output file.
  * Returns:
  *    The number of bytes transfered or -1 on an error.
- *
  *-----------------------------------------------------------------------------
  */
 static int
@@ -154,10 +169,14 @@ CopyOpenFile (interp, maxBytes, inFilePtr, outFilePtr)
 
         bytesRead = fread (buffer, sizeof (char), bytesToRead, inFilePtr);
         if (bytesRead <= 0) {
-            if (feof (inFilePtr))
+            if (feof (inFilePtr)) {
                 break;
-            else
-                goto unixError;
+            }
+            if (((errno == EWOULDBLOCK) || (errno == EAGAIN)) &&
+                (totalBytesRead > 0)) {
+                break;  /* Would blocking, but got some data. */
+            }
+            goto unixError;
         }
         if (fwrite (buffer, sizeof (char), bytesRead, outFilePtr) != bytesRead)
             goto unixError;
@@ -200,7 +219,7 @@ Tcl_CopyfileCmd (clientData, interp, argc, argv)
 #define TCL_COPY_MAX_BYTES  2
 
     FILE  *inFilePtr, *outFilePtr;
-    long   bytesToRead, bytesRead, totalBytesToRead, totalBytesRead, bytesLeft;
+    long   totalBytesToRead, totalBytesRead;
     int    copyMode;
 
     if (!(argc == 3 || argc == 5))
@@ -610,7 +629,6 @@ Tcl_FrenameCmd (clientData, interp, argc, argv)
                           (char *) NULL);
         return TCL_ERROR;
     }
-
     
     Tcl_DStringFree (&tildeBuf1);
     Tcl_DStringFree (&tildeBuf2);
@@ -622,6 +640,151 @@ Tcl_FrenameCmd (clientData, interp, argc, argv)
     return TCL_ERROR;
 }
 
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * TruncateByPath --
+ * 
+ *  Truncate a file via path, if this is available on this system.
+ *
+ * Parameters:
+ *   o interp (I) - Error messages are returned in the interpreter.
+ *   o filePath (I) - Path to file.
+ *   o newSize (I) - Size to truncate the file to.
+ * Returns:
+ *   TCL_OK or TCL_ERROR.
+ *-----------------------------------------------------------------------------
+ */
+static int
+TruncateByPath (interp, filePath, newSize)
+    Tcl_Interp  *interp;
+    char        *filePath;
+    off_t        newSize;
+{
+#if defined(HAVE_TRUNCATE)
+    Tcl_DString  tildeBuf;
+
+    Tcl_DStringInit (&tildeBuf);
+
+    filePath = Tcl_TildeSubst (interp, filePath, &tildeBuf);
+    if (filePath == NULL) {
+        Tcl_DStringFree (&tildeBuf);
+        return TCL_ERROR;
+    }
+    if (truncate (filePath, newSize) != 0) {
+        Tcl_AppendResult (interp, filePath, ": ", Tcl_PosixError (interp), (char *) NULL);
+        Tcl_DStringFree (&tildeBuf);
+        return TCL_ERROR;
+    }
+
+    Tcl_DStringFree (&tildeBuf);
+    return TCL_OK;
+#else
+    Tcl_AppendResult (interp, "truncating files by path is not available on this system",
+                      (char *) NULL);
+    return TCL_ERROR;
+#endif
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * TruncateByHandle --
+ * 
+ *  Truncate a file via a open file handle., if this is available on this system.
+ *
+ * Parameters:
+ *   o interp (I) - Error messages are returned in the interpreter.
+ *   o fileHandle (I) - Path to file.
+ *   o newSize (I) - Size to truncate the file to.
+ * Returns:
+ *   TCL_OK or TCL_ERROR.
+ *-----------------------------------------------------------------------------
+ */
+static int
+TruncateByHandle (interp, fileHandle, newSize)
+    Tcl_Interp  *interp;
+    char        *fileHandle;
+    off_t        newSize;
+{
+#if defined(HAVE_FTRUNCATE) || defined(HAVE_CHSIZE)
+    int   stat;
+    FILE *filePtr;
+
+    if (Tcl_GetOpenFile (interp, fileHandle,
+                         TRUE, TRUE, /* Write access */
+                         &filePtr) != TCL_OK)
+        return TCL_ERROR;
+
+#ifdef HAVE_FTRUNCATE
+    stat = ftruncate (fileno (filePtr), newSize);
+#else
+    stat = chsize (fileno (filePtr), newSize);
+#endif
+    if (stat != 0) {
+        Tcl_AppendResult (interp, fileHandle, ": ", Tcl_PosixError (interp), (char *) NULL);
+        return TCL_ERROR;
+    }
+    return TCL_OK;
+#else
+    Tcl_AppendResult (interp, FILE_ID_NOT_AVAIL, (char *) NULL);
+    return TCL_ERROR;
+#endif
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Tcl_FtruncateCmd --
+ *     Implements the Tcl ftruncate command:
+ *     ftruncate [-fileid] file newsize
+ *
+ * Results:
+ *  Standard TCL results, may return the UNIX system error message.
+ *
+ *-----------------------------------------------------------------------------
+ */
+int
+Tcl_FtruncateCmd (clientData, interp, argc, argv)
+    ClientData   clientData;
+    Tcl_Interp  *interp;
+    int          argc;
+    char       **argv;
+{
+    int    argIdx, fileIds;
+    off_t  newSize;
+
+    fileIds = FALSE;
+    for (argIdx = 1; (argIdx < argc) && (argv [argIdx] [0] == '-'); argIdx++) {
+        if (STREQU (argv [argIdx], FILE_ID_OPT)) {
+            fileIds = TRUE;
+        } else {
+            Tcl_AppendResult (interp, "Invalid option \"", argv [argIdx],
+                              "\", expected \"", FILE_ID_OPT, "\"",
+                              (char *) NULL);
+            return TCL_ERROR;
+        }
+    }
+
+    if (argIdx != argc - 2) {
+        Tcl_AppendResult (interp, tclXWrongArgs, argv [0], 
+                          " [-fileid] file newsize", (char *) NULL);
+        return TCL_ERROR;
+    }
+
+    if (Tcl_GetOffset (interp,
+                       argv [argIdx + 1],
+                       &newSize) != TCL_OK)
+        return TCL_ERROR;
+
+
+    if (fileIds) {
+        return TruncateByHandle (interp, argv [argIdx], newSize);
+    } else {
+        return TruncateByPath (interp, argv [argIdx], newSize);
+    }
+}
 
 /*
  *-----------------------------------------------------------------------------
