@@ -3,7 +3,7 @@
  *
  *  Extended Tcl keyed list commands and interfaces.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1993 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1995 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXkeylist.c,v 3.0 1993/11/19 06:58:51 markd Rel $
+ * $Id: tclXkeylist.c,v 4.3 1995/01/01 19:49:31 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -196,7 +196,7 @@ SplitAndFindField (interp, fieldName, keyedList, fieldInfoPtr)
 
 errorExit:
     if (fieldInfoPtr->argv != NULL)
-        ckfree (fieldInfoPtr->argv);
+        ckfree ((char *) fieldInfoPtr->argv);
     fieldInfoPtr->argv = NULL;
     return TCL_ERROR;
 }
@@ -241,9 +241,19 @@ Tcl_GetKeyedListKeys (interp, subFieldName, keyedList, keyesArgcPtr,
     int    fieldSize,  keySize;
     char **keyArgv;
 
+    /*
+     * Skip leading white spaces in list.  This keeps totally empty lists
+     * with some white-spaces from being confused with empty field entries
+     * later on in the parsing.
+     */
+    for (; *keyedList != '\0'; keyedList++) {
+        if (ISSPACE (*keyedList) == 0)
+            break;
+    }
+
     /*
      * If the keys of a subfield are requested, the dig out that field's
-     * list and then rummage through in getting the keys.
+     * list and then rummage through it getting the keys.
      */
     subFieldList = NULL;
     if ((subFieldName != NULL) && (subFieldName [0] != '\0')) {
@@ -351,6 +361,16 @@ Tcl_GetKeyedListField (interp, fieldName, keyedList, fieldValuePtr)
         return TCL_ERROR;
     }
 
+    /*
+     * Skip leading white spaces in list.  This keeps totally empty lists
+     * with some white-spaces from being confused with empty field entries
+     * later on in the parsing.
+     */
+    for (; *keyedList != '\0'; keyedList++) {
+        if (ISSPACE (*keyedList) == 0)
+            break;
+    }
+
     /*
      * Check for sub-names, temporarly delimit the top name with a '\0'.
      */
@@ -450,6 +470,11 @@ Tcl_SetKeyedListField (interp, fieldName, fieldValue, keyedList)
     fieldInfo_t  fieldInfo;
     char        *elemArgv [2];
 
+    if (fieldName [0] == '\0') {
+        Tcl_AppendResult (interp, "empty field name", (char *) NULL);
+        return NULL;
+    }
+
     if (keyedList == NULL)
         keyedList = "";
 
@@ -634,8 +659,6 @@ Tcl_KeyldelCmd (clientData, interp, argc, argv)
     char      **argv;
 {
     char  *keyedList, *newList;
-    int    listArgc, fieldIdx, idx;
-    char **listArgv;
     char  *varPtr;
 
     if (argc != 3) {
@@ -802,7 +825,7 @@ Tcl_KeylkeysCmd (clientData, interp, argc, argv)
     }
 
     Tcl_SetResult (interp, Tcl_Merge (keyesArgc, keyesArgv), TCL_DYNAMIC);
-    ckfree (keyesArgv);
+    ckfree ((char *) keyesArgv);
     return TCL_OK;
 }
 
