@@ -3,7 +3,7 @@
  *
  * Tcl commands to load libraries of Tcl code.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1993 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1995 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXlib.c,v 3.3 1994/01/05 06:10:20 markd Exp $
+ * $Id: tclXlib.c,v 4.5 1995/01/16 07:39:53 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -44,6 +44,14 @@ static char *AUTO_PATH      = "auto_path";
 static char *AUTO_OLDPATH   = "auto_oldpath";
 static char *AUTO_PKG_INDEX = "auto_pkg_index";
 
+/*
+ * Command to pass to Tcl_GlobalEval to load the file loadouster.tcl.
+ * This is a global rather than a local so it will work with K&R compilers.
+ * Its writable so it works with gcc.
+ */
+static char loadOusterCmd [] = "source [info library]/loadouster.tcl";
+
+
 /*
  * Per-interpreter structure used for managing the library.
  */
@@ -56,13 +64,9 @@ typedef struct libInfo_t {
  * Prototypes of internal functions.
  */
 static int
-GlobalEvalFile _ANSI_ARGS_((Tcl_Interp *interp,
-                            char       *file));
-
-static int
 EvalFilePart _ANSI_ARGS_((Tcl_Interp  *interp,
                           char        *fileName,
-                          long         offset,
+                          off_t        offset,
                           unsigned     length));
 
 static char *
@@ -81,7 +85,7 @@ static int
 GetPackageIndexEntry _ANSI_ARGS_((Tcl_Interp *interp,
                                   char       *packageName,
                                   char      **fileNamePtr,
-                                  long       *offsetPtr,
+                                  off_t      *offsetPtr,
                                   unsigned   *lengthPtr));
 
 static int
@@ -141,29 +145,6 @@ static void
 TclLibCleanUp _ANSI_ARGS_((ClientData  clientData,
                            Tcl_Interp *interp));
 
-/*
- *-----------------------------------------------------------------------------
- * GlobalEvalFile --
- *
- *  Evaluate a file at global level in an interpreter.
- *-----------------------------------------------------------------------------
- */
-static int
-GlobalEvalFile(interp, file)
-    Tcl_Interp *interp;
-    char       *file;
-{
-    register Interp *iPtr = (Interp *) interp;
-    int result;
-    CallFrame *savedVarFramePtr;
-
-    savedVarFramePtr = iPtr->varFramePtr;
-    iPtr->varFramePtr = NULL;
-    result = Tcl_EvalFile (interp, file);
-    iPtr->varFramePtr = savedVarFramePtr;
-    return result;
-}
-
 /*
  *-----------------------------------------------------------------------------
  * EvalFilePart --
@@ -181,7 +162,7 @@ static int
 EvalFilePart (interp, fileName, offset, length)
     Tcl_Interp  *interp;
     char        *fileName;
-    long         offset;
+    off_t        offset;
     unsigned     length;
 {
     Interp       *iPtr = (Interp *) interp;
@@ -190,6 +171,7 @@ EvalFilePart (interp, fileName, offset, length)
     char         *oldScriptFile, *cmdBuffer, *buf;
     Tcl_DString   tildeBuf;
 
+    Tcl_ResetResult (interp);
     Tcl_DStringInit (&tildeBuf);
     
     if (fileName [0] == '~') {
@@ -242,7 +224,7 @@ EvalFilePart (interp, fileName, offset, length)
      * An error occured in the command, record information telling where it
      * came from.
      */
-    buf = ckalloc (sizeof (fileName) + 64);
+    buf = ckalloc (strlen (fileName) + 64);
     sprintf (buf, "\n    (file \"%s\" line %d)", fileName,
              interp->errorLine);
     Tcl_AddErrorInfo (interp, buf);
@@ -396,10 +378,10 @@ GetPackageIndexEntry (interp, packageName, fileNamePtr, offsetPtr, lengthPtr)
      Tcl_Interp *interp;
      char       *packageName;
      char      **fileNamePtr;
-     long       *offsetPtr;
+     off_t       *offsetPtr;
      unsigned   *lengthPtr;
 {
-    int   pkgDataArgc, idx;
+    int   pkgDataArgc;
     char *dataStr, **pkgDataArgv = NULL;
     char *srcPtr, *destPtr;
 
@@ -424,7 +406,7 @@ GetPackageIndexEntry (interp, packageName, fileNamePtr, offsetPtr, lengthPtr)
     if (pkgDataArgc != 3)
         goto invalidEntry;
 
-    if (!Tcl_StrToLong (pkgDataArgv [1], 0, offsetPtr))
+    if (!Tcl_StrToOffset (pkgDataArgv [1], 0, offsetPtr))
         goto invalidEntry;
     if (!Tcl_StrToUnsigned (pkgDataArgv [2], 0, lengthPtr))
         goto invalidEntry;
@@ -444,7 +426,7 @@ GetPackageIndexEntry (interp, packageName, fileNamePtr, offsetPtr, lengthPtr)
      */
   invalidEntry:
     if (pkgDataArgv != NULL)
-        ckfree (pkgDataArgv);
+        ckfree ((char *) pkgDataArgv);
     Tcl_ResetResult (interp);
     Tcl_AppendResult (interp, "invalid entry in \"auto_pkg_index \"",
                       "for package \"", packageName, "\"", (char *) NULL);
@@ -583,7 +565,7 @@ ProcessIndexFile (interp, tlibFilePath, tndxFilePath)
                     goto errorExit;
             }
         }
-        ckfree (lineArgv);
+        ckfree ((char *) lineArgv);
         lineArgv = NULL;
     }
 
@@ -615,7 +597,7 @@ ProcessIndexFile (interp, tlibFilePath, tndxFilePath)
      */
   errorExit:
     if (lineArgv != NULL)
-        ckfree (lineArgv);
+        ckfree ((char *) lineArgv);
     Tcl_DStringFree (&lineBuffer);
     if (indexFilePtr != NULL)
         fclose (indexFilePtr);
@@ -738,8 +720,8 @@ LoadPackageIndex (interp, tlibFilePath)
  *   The real work of loading an index is done by a procedure that is defined
  * in a seperate file.  Its not possible to put this file in the standard
  * tcl.tlib as a tclIndex might get loaded before the tcl.tndx file is found
- * on the search path.  The function sets up the auto_index array to load the
- * procedure if its not already defined. 
+ * on the search path.  The function sources [info library]/loadouster.tcl
+ * if it has not been loaded before.
  *-----------------------------------------------------------------------------
  */
 static int
@@ -748,24 +730,38 @@ LoadOusterIndex (interp, indexFilePath)
      char       *indexFilePath;
 {
     Tcl_DString  command;
+    Tcl_CmdInfo  loadCmdInfo;
     
     Tcl_DStringInit (&command);
-    Tcl_DStringAppend (&command, "set auto_index(auto_load_ouster_index) ",
-                       -1);
-    Tcl_DStringAppend (&command, "\"source [info library]/loadouster.tcl\";",
-                       -1);
-    Tcl_DStringAppend (&command, "auto_load_ouster_index {", -1);
-    Tcl_DStringAppend (&command, indexFilePath, -1);
-    Tcl_DStringAppend (&command, "}", -1);
 
-    if (Tcl_GlobalEval (interp, command.string) == TCL_ERROR) {
-        AddLibIndexErrorInfo (interp, indexFilePath);
-        Tcl_DStringFree (&command);
-        return TCL_ERROR;
+    /*
+     * Check if "auto_load_ouster_index" is defined, if its not, source
+     * the file that defines it.
+     */
+    if (!Tcl_GetCommandInfo (interp,
+                             "auto_load_ouster_index",
+                             &loadCmdInfo)) {
+        if (Tcl_GlobalEval (interp, loadOusterCmd) == TCL_ERROR)
+            goto errorExit;
     }
+
+    /*
+     * Now, actually do the load.
+     */
+    Tcl_DStringAppendElement (&command, "auto_load_ouster_index");
+    Tcl_DStringAppendElement (&command, indexFilePath);
+
+    if (Tcl_GlobalEval (interp, command.string) == TCL_ERROR)
+        goto errorExit;
+
     Tcl_DStringFree (&command);
     Tcl_ResetResult (interp);
     return TCL_OK;
+
+  errorExit:
+    AddLibIndexErrorInfo (interp, indexFilePath);
+    Tcl_DStringFree (&command);
+    return TCL_ERROR;
 }
 
 /*
@@ -902,7 +898,7 @@ LoadPackageIndexes (interp, infoPtr, path)
             break;
     }
 
-    ckfree (pathArgv);
+    ckfree ((char *) pathArgv);
     infoPtr->doingIdxSearch = FALSE;
     return result;
 }
@@ -925,7 +921,7 @@ Tcl_Auto_load_pkgCmd (dummy, interp, argc, argv)
     char       **argv;
 {
     char     *fileName;
-    long      offset;
+    off_t     offset;
     unsigned  length;
     int       result;
 
@@ -1069,10 +1065,8 @@ LoadCommand (interp, command)
     Tcl_Interp  *interp;
     char        *command;
 {
-    char              *loadCmd;
-    ClientData         clientData;
-    Tcl_CmdDeleteProc *deleteProc;
-    Tcl_CmdInfo        cmdInfo;
+    char        *loadCmd;
+    Tcl_CmdInfo  cmdInfo;
 
     loadCmd = Tcl_GetVar2 (interp, AUTO_INDEX, command, TCL_GLOBAL_ONLY);
     if (loadCmd == NULL)
@@ -1274,13 +1268,13 @@ TclLibCleanUp (clientData, interp)
 
 /*
  *-----------------------------------------------------------------------------
- * TclXLib_Init --
+ * TclX_LibraryInit --
  *
  *   Initialize the Extended Tcl library facility commands.
  *-----------------------------------------------------------------------------
  */
 int
-TclXLib_Init (interp)
+TclX_LibraryInit (interp)
     Tcl_Interp *interp;
 {
     libInfo_t *infoPtr;
