@@ -3,7 +3,7 @@
  *
  * Tcl regular expression pattern matching utilities.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1993 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1995 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -16,26 +16,31 @@
  *     torek-boyer-moore/27-Aug-90 by
  *     chris@mimsy.umd.edu (Chris Torek)
  *-----------------------------------------------------------------------------
- * $Id: tclXregexp.c,v 3.0 1993/11/19 06:59:11 markd Rel $
+ * $Id: tclXregexp.c,v 5.0 1995/07/25 05:42:54 markd Rel $
  *-----------------------------------------------------------------------------
  */
 
 #include "tclExtdInt.h"
 
 /*
- * This is declared in tclUtil.c.  Must be set to NULL before compiling
- * a regular expressions.
+ * Structure used to return pre-parse infomation about a regular expression.
  */
-extern char *tclRegexpError;
+typedef struct {
+    int   meta;
+    int   numSubExprs;
+    char *largestNonMeta;
+    int   largestNonMetaLen;
+} preParseInfo_t;
+
 
 /*
  * Meta-characters for regular expression
  */
-#define REXP_META                   "^$.[()|?+*\\"
-#define REXP_META_NO_BRACKET_NO_OR  "^$.()?+*\\"
+#define TCLX_REXP_META                   "^$.[()|?+*\\"
+#define TCLX_REXP_META_NO_BRACKET_NO_OR  "^$.()?+*\\"
 
-#ifndef CHAR_MAX
-#    define CHAR_MAX 255
+#ifndef UCHAR_MAX
+#    define UCHAR_MAX 255
 #endif
 
 /*
@@ -53,8 +58,8 @@ BoyerMooreExecute _ANSI_ARGS_((char     *text,
                                unsigned *patLenP));
 
 static int
-FindNonRegExpSubStr _ANSI_ARGS_((char  *expression,
-                                 char **subStrPtrPtr));
+PreParseRegExp _ANSI_ARGS_((char           *expression,
+                            preParseInfo_t *infoPtr));
 
 
 /*
@@ -169,7 +174,7 @@ FindNonRegExpSubStr _ANSI_ARGS_((char  *expression,
 
 struct compiled_search_struct {
         unsigned patlen;
-        unsigned deltaspace[CHAR_MAX + 1];
+        unsigned deltaspace[UCHAR_MAX + 1];
 };
 
 
@@ -178,8 +183,8 @@ BoyerMooreCompile (pat, patlen)
     char *pat;
     int   patlen;
 {
-        register unsigned char *p, *t;
-        register unsigned i, p1, j, *delta;
+        register unsigned char *p;
+        register unsigned i, p1, *delta;
         struct compiled_search_struct *cp;
         int alloc_len;
 
@@ -198,7 +203,7 @@ BoyerMooreCompile (pat, patlen)
         /* set up deltas */
         delta = cp->deltaspace;
 
-        for (i = 0; i <= CHAR_MAX; i++)
+        for (i = 0; i <= UCHAR_MAX; i++)
                 delta[i] = p1;
 
         for (p = (unsigned char *)pat, i = p1; --i > 0;)
@@ -257,15 +262,15 @@ BoyerMooreExecute (text, textlen, compPtr, patLenP)
 /*
  *-----------------------------------------------------------------------------
  *
- * Tcl_RegExpClean --
+ * TclX_RegExpClean --
  *     Free all resources associated with a regular expression info 
  *     structure..
  *
  *-----------------------------------------------------------------------------
  */
 void
-Tcl_RegExpClean (regExpPtr)
-    Tcl_regexp *regExpPtr;
+TclX_RegExpClean (regExpPtr)
+    TclX_regexp *regExpPtr;
 {
     if (regExpPtr->progPtr != NULL)
     	ckfree ((char *) regExpPtr->progPtr);
@@ -276,68 +281,142 @@ Tcl_RegExpClean (regExpPtr)
 /*
  *-----------------------------------------------------------------------------
  *
- * FindNonRegExpSubStr
- *     Find the largest substring that does not have any regular 
- *     expression meta-characters and is not located within `[...]'.
- *     If the regexp contains an or (|), zero is returned, as the 
- *     Boyer-Moore optimization does not work, since there are actually
- *     multiple patterns.  The real solution is to build the Boyer-Moore
- *     into the regular expression code.
+ * PreParseRegExp --
+ *
+ *   Do preparsing of the regular expression to allow us to do various
+ * optimizations outside of the regular expression parsing.  This includes
+ * detecting if there are any regular expression meta-characters and what
+ * the largest substring is for boyer-moore matching.
+ *
+ * Parameters:
+ *   o expression (I) - Expression to parse.
+ *   o infoPtr (O) - Information is returned in this structure.  The following
+ *     fields are returned:
+ *     o meta - TRUE if there are any meta characters, FALSE otherwise.
+ *     o numSubExprs - The number of subexpressions in the expression.
+ *     o largestNonMeta - Pointer to the beginning of the largest string
+ *       without any meta characters.
+ *     o largestNonMetaLen - Length of the largest non-meta character string.
+ * Returns:
+ *    TRUE if the parse succeeded, FALSE if there appears to be an error.
+ * Call TclRegComp to get a real error message.
  *-----------------------------------------------------------------------------
  */
 static int
-FindNonRegExpSubStr (expression, subStrPtrPtr)
-    char  *expression;
-    char **subStrPtrPtr;
+PreParseRegExp (expression, infoPtr)
+    char           *expression;
+    preParseInfo_t *infoPtr;
 {
-    register char *subStrPtr = NULL;
-    register char  subStrLen = 0;
-    register char *scanPtr   = expression;
-    register int   len;
+    register char *scanPtr  = expression;
+    register char *nonMetaStart;
+    register int   nonMetaCnt, gotMeta, parenNest;
 
+    infoPtr->meta = FALSE;
+    infoPtr->numSubExprs = 0;
+    infoPtr->largestNonMeta = NULL;
+    infoPtr->largestNonMetaLen = 0;
+
+    nonMetaCnt = 0;
+
+    /*
+     * Scan counting subexpressions and searching for the longest string
+     * containing no meta-characters.  Note that backslashed characters
+     * are considered meta for this purpose as they will not behave as
+     * exprected in the Boyer-Moore matching code.  Strings in subexpressions
+     * are not counted.
+     */
     while (*scanPtr != '\0') {
-        len = strcspn (scanPtr, REXP_META);
-        /*
-         * If we are at a meta-character, by-pass till non-meta.  If we hit
-         * a `[' then by-pass the entire `[...]' range, but be careful, could
-         * have omitted `]'.  In a `|' is encountered (except in brackets),'
-         * we are through.
-         */
-        if (len == 0) {
-            scanPtr += strspn (scanPtr, REXP_META_NO_BRACKET_NO_OR);
-            if (*scanPtr == '|')
-                return 0;
-            if (*scanPtr == '[') {
-                scanPtr += strcspn (scanPtr, "]");
-                if (*scanPtr == ']')
-                    scanPtr++;
-            }          
-        } else {
-            if (len > subStrLen) {
-                subStrPtr = scanPtr;
-                subStrLen = len;
+        switch (*scanPtr++) {
+          case '^':
+          case '$':
+          case '.':
+          case '?':
+          case '+':
+          case '*':
+            gotMeta = TRUE;
+            break;
+          case '[':
+            gotMeta = TRUE;
+            while (*scanPtr != ']' && *scanPtr != '\0') {
+                scanPtr++;
             }
-            scanPtr += len;
+            if (*scanPtr == '\0')
+                return FALSE;
+            break;
+          case '(':
+            gotMeta = TRUE;
+            infoPtr->numSubExprs++;
+            parenNest = 1;
+            while (parenNest > 0 && *scanPtr != '\0') {
+                switch (*scanPtr++) {
+                  case '\\':
+                    if (scanPtr == '\0')
+                        return FALSE;
+                    scanPtr++;
+                    break;
+                  case '(':
+                    infoPtr->numSubExprs++;
+                    parenNest++;
+                    break;
+                  case ')':
+                    parenNest--;
+                    break;
+                }
+            }
+            if ((*scanPtr == '\0') && (parenNest > 0))
+                return FALSE;
+            break;
+          case '\\':
+            gotMeta = TRUE;
+            if (*scanPtr == '\0')
+                return FALSE;
+            scanPtr++;
+            break;
+          default:
+            gotMeta = FALSE;
+            if (nonMetaCnt == 0)
+                nonMetaStart = scanPtr - 1;
+            nonMetaCnt++;
+        }
+        /*
+         * If we just hit a meta character, save the non-meta substring
+         * if its the longest one we have hit.
+         */
+        if (gotMeta && (nonMetaCnt > infoPtr->largestNonMetaLen)) {
+                infoPtr->largestNonMeta = nonMetaStart;
+                infoPtr->largestNonMetaLen = nonMetaCnt;
+        }
+        if (gotMeta) {
+            infoPtr->meta = TRUE;
+            nonMetaCnt = 0;
         }
     }
-    *subStrPtrPtr = subStrPtr;
-    return subStrLen;
+
+    /*
+     * Handle counting non-meta substring at the end of the string.
+     */
+    if (nonMetaCnt > infoPtr->largestNonMetaLen) {
+        infoPtr->largestNonMeta = nonMetaStart;
+        infoPtr->largestNonMetaLen = nonMetaCnt;
+    }
+
+    return TRUE;
 }
 
 /*
  *-----------------------------------------------------------------------------
  *
- * Tcl_RegExpCompile --
+ * TclX_RegExpCompile --
  *     Compile a regular expression.
  *
  * Parameters:
  *     o regExpPtr - Used to hold info on this regular expression.  If the
- *       structure is being reused, it Tcl_RegExpClean should be called first.
+ *       structure is being reused, it TclX_RegExpClean should be called first.
  *     o expression - Regular expression to compile.
  *     o flags - The following flags are recognized:
- *         o REXP_NO_CASE - Comparison will be regardless of case.
- *         o REXP_BOTH_ALGORITHMS - If specified, a Boyer-Moore expression is 
- *           compiled for the largest substring of the expression that does
+ *         o TCLX_REXP_NO_CASE - Comparison will be regardless of case.
+ *         o TCLX_REXP_BOTH_ALGORITHMS - If specified, a Boyer-Moore expression
+ *           is compiled for the largest substring of the expression that does
  *           not contain any meta-characters.  This is slows compiling, but
  *           speeds up large searches.
  *
@@ -346,14 +425,15 @@ FindNonRegExpSubStr (expression, subStrPtrPtr)
  *-----------------------------------------------------------------------------
  */
 int
-Tcl_RegExpCompile (interp, regExpPtr, expression, flags)
+TclX_RegExpCompile (interp, regExpPtr, expression, flags)
     Tcl_Interp  *interp;
-    Tcl_regexp  *regExpPtr;
+    TclX_regexp *regExpPtr;
     char        *expression;
     int          flags;
 {
-    char *expBuf;
-    int   anyMeta;
+    char           *expBuf;
+    int             preParseOk;
+    preParseInfo_t  preParseInfo;
 
     if (*expression == '\0') {
         Tcl_AppendResult (interp, "Null regular expression", (char *) NULL);
@@ -362,66 +442,73 @@ Tcl_RegExpCompile (interp, regExpPtr, expression, flags)
 
     regExpPtr->progPtr = NULL;
     regExpPtr->boyerMoorePtr = NULL;
-    regExpPtr->noCase = flags & REXP_NO_CASE;
+    regExpPtr->noCase = flags & TCLX_REXP_NO_CASE;
 
-    if (flags & REXP_NO_CASE) {
+    if (flags & TCLX_REXP_NO_CASE) {
         expBuf = ckalloc (strlen (expression) + 1);
         Tcl_DownShift (expBuf, expression);
-    } else
+    } else {
         expBuf = expression;
-
-    anyMeta = strpbrk (expBuf, REXP_META) != NULL;
+    }
 
     /*
-     * If no meta-characters, use Boyer-Moore string matching only.
+     * Pre-parse the expression to gain information used for Boyer-Moore
+     * optimization and returning subexpression results.  If an error occurs,
+     * force the expression compilation anyway to get a real error message.
      */
-    if ((!anyMeta) && (flags & REXP_BOTH_ALGORITHMS)) {
-        regExpPtr->boyerMoorePtr = BoyerMooreCompile (expBuf, strlen (expBuf));
-        goto okExitPoint;
+    preParseOk = PreParseRegExp (expBuf,
+                                 &preParseInfo);
+    if (!preParseOk) {
+        preParseInfo.meta = TRUE;
+        preParseInfo.largestNonMetaLen = -1;
     }
- 
+    regExpPtr->numSubExprs = preParseInfo.numSubExprs;
+
     /*
      * Build a Boyer-Moore on the largest non-meta substring, if requested,
-     * and the reg-exp does not contain a `|' (or).  If less that three
-     * characters in the string, don't use B-M, as it seems not optimal at
-     * this point.
+     * If less than three characters in the string and there are meta 
+     * characters, don't use B-M, as it seems not optimal at this point.
      */
-    if (flags & REXP_BOTH_ALGORITHMS) {
-        char *subStrPtr;
-        int   subStrLen;
-        
-        subStrLen = FindNonRegExpSubStr (expBuf, &subStrPtr);
-        if (subStrLen > 2)
+    if ((flags & TCLX_REXP_BOTH_ALGORITHMS) && 
+        (preParseInfo.largestNonMetaLen >= (preParseInfo.meta ? 3 : 0))) {
             regExpPtr->boyerMoorePtr = 
-                BoyerMooreCompile (subStrPtr, subStrLen);
+                BoyerMooreCompile (preParseInfo.largestNonMeta,
+                                   preParseInfo.largestNonMetaLen);
     }
     
     /*
-     * Compile meta-character containing regular expression.
+     * Compile meta-character containing regular expression or ones with
+     * errors to generate a useful error message.
      */
-    tclRegexpError = NULL;
-    regExpPtr->progPtr = TclRegComp (expBuf);
-    if (tclRegexpError != NULL) {
-        if (flags & REXP_NO_CASE)
-            ckfree (expBuf);
-        Tcl_AppendResult (interp, "error in regular expression: ", 
-                          tclRegexpError, (char *) NULL);
-        if (flags & REXP_NO_CASE)
-            ckfree (expBuf);
-        Tcl_RegExpClean (regExpPtr);
+    if (preParseInfo.meta) {
+        regExpPtr->progPtr = TclRegComp (expBuf);
+        
+        /*
+         * If the preparsing reported an error, but the compile didn't,
+         * we have a bug in the pre-parser.
+         */
+        if ((!preParseOk) && (regExpPtr->progPtr != NULL))
+            panic ("scanmatch preparse bug");
+        
+        if (regExpPtr->progPtr == NULL) {
+            Tcl_AppendResult (interp, "error in regular expression: ", 
+                              TclGetRegError (), (char *) NULL);
+            if (flags & TCLX_REXP_NO_CASE)
+                ckfree (expBuf);
+            TclX_RegExpClean (regExpPtr);
+            return TCL_ERROR;
+        }
     }
-  
-okExitPoint: 
-    if (flags & REXP_NO_CASE)
+
+    if (flags & TCLX_REXP_NO_CASE)
         ckfree (expBuf);
     return TCL_OK;
-
 }
 
 /*
  *-----------------------------------------------------------------------------
  *
- * Tcl_RegExpExecute --
+ * TclX_RegExpExecute --
  *     Execute a regular expression compiled with Boyer-Moore and/or 
  *     regexp.
  *
@@ -440,9 +527,9 @@ okExitPoint:
  *-----------------------------------------------------------------------------
  */
 int
-Tcl_RegExpExecute (interp, regExpPtr, matchStrIn, matchStrLower, subMatchInfo)
+TclX_RegExpExecute (interp, regExpPtr, matchStrIn, matchStrLower, subMatchInfo)
     Tcl_Interp       *interp;
-    Tcl_regexp       *regExpPtr;
+    TclX_regexp      *regExpPtr;
     char             *matchStrIn;
     char             *matchStrLower;
     Tcl_SubMatchInfo  subMatchInfo;
@@ -480,8 +567,6 @@ Tcl_RegExpExecute (interp, regExpPtr, matchStrIn, matchStrLower, subMatchInfo)
          * If no regexp, its a match!
          */
         if (regExpPtr->progPtr == NULL) {
-            subMatchInfo [0].start = -1;
-            subMatchInfo [0].end = -1;
             result = TRUE; 
             goto exitPoint;
         }
@@ -497,14 +582,17 @@ Tcl_RegExpExecute (interp, regExpPtr, matchStrIn, matchStrLower, subMatchInfo)
      * Return submatches if we found it.
      */
     if (result) {
-        for (idx = 1; idx < NSUBEXP; idx++) {
-            if (progPtr->startp [idx] == NULL)
-                break;
-            subMatchInfo [idx - 1].start = progPtr->startp [idx] - matchStr;
-            subMatchInfo [idx - 1].end = progPtr->endp [idx] - matchStr - 1;
+        for (idx = 1; idx < regExpPtr->numSubExprs + 1; idx++) {
+            if (progPtr->startp [idx] == NULL) {
+                subMatchInfo [idx - 1].start = -1;
+                subMatchInfo [idx - 1].end = -1;
+            } else {
+                subMatchInfo [idx - 1].start =
+                    progPtr->startp [idx] - matchStr;
+                subMatchInfo [idx - 1].end =
+                    progPtr->endp [idx] - matchStr - 1;
+            }
         }
-        subMatchInfo [idx - 1].start = -1;
-        subMatchInfo [idx - 1].end = -1;
     }
 
     /*
