@@ -3,7 +3,7 @@
  *
  * Tcl commands to access unix library calls.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1993 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1995 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXunixcmds.c,v 3.0 1993/11/19 06:59:25 markd Rel $
+ * $Id: tclXunixcmds.c,v 5.0 1995/07/25 05:58:58 markd Rel $
  *-----------------------------------------------------------------------------
  */
 
@@ -192,18 +192,21 @@ Tcl_NiceCmd (clientData, interp, argc, argv)
     if (Tcl_GetInt (interp, argv[1], &priorityIncr) != TCL_OK)
         return TCL_ERROR;
 
-    errno = 0;  /* In case old priority is -1 */
+    errno = 0;  /* Old priority might be -1 */
 
+#ifdef HAVE_GETPRIORITY
+    priority = getpriority (PRIO_PROCESS, 0) + priorityIncr;
+    if (errno == 0) {
+        setpriority (PRIO_PROCESS, 0, priority);
+    }
+#else
     priority = nice (priorityIncr);
-    if ((priority  == -1) && (errno != 0)) {
+#endif
+    if (errno != 0) {
         interp->result = Tcl_PosixError (interp);
         return TCL_ERROR;
     }
 
-#ifdef HAVE_GETPRIORITY
-    priority = getpriority (PRIO_PROCESS, 0);
-#endif
-
     sprintf (interp->result, "%d", priority);
     return TCL_OK;
 }
@@ -262,8 +265,7 @@ Tcl_SyncCmd (clientData, interp, argc, argv)
     int         argc;
     char      **argv;
 {
-    unsigned time;
-    FILE *filePtr;
+    FILE  *filePtr;
 
     if ((argc < 1) || (argc > 2)) {
         Tcl_AppendResult (interp, tclXWrongArgs, argv [0], " ?filehandle?",
@@ -314,7 +316,7 @@ Tcl_SystemCmd (clientData, interp, argc, argv)
     int         argc;
     char      **argv;
 {
-    int exitCode;
+    int  waitStatus;
 
     if (argc != 2) {
         Tcl_AppendResult (interp, tclXWrongArgs, argv [0], " command",
@@ -322,13 +324,37 @@ Tcl_SystemCmd (clientData, interp, argc, argv)
         return TCL_ERROR;
     }
 
-    exitCode = system (argv [1]);
-    if (exitCode == -1) {
+    waitStatus = system (argv [1]);
+    if (waitStatus == -1) {
         interp->result = Tcl_PosixError (interp);
         return TCL_ERROR;
     }
-    sprintf (interp->result, "%d", exitCode);
-    return TCL_OK;
+    if (WIFEXITED (waitStatus)) {
+        sprintf (interp->result, "%d", WEXITSTATUS (waitStatus));
+        return TCL_OK;
+    }
+
+    /*
+     * Return status based on wait result.
+     */
+    if (WIFSIGNALED (waitStatus)) {
+        Tcl_SetErrorCode (interp, "SYSTEM", "SIG",
+                          Tcl_SignalId (WTERMSIG (waitStatus)), (char *) NULL);
+        Tcl_AppendResult (interp, "system command terminate with signal ",
+                          Tcl_SignalId (WTERMSIG (waitStatus)), (char *) NULL);
+        return TCL_ERROR;
+    }
+
+    /*
+     * Should never get this status back unless the implementation is
+     * really brain-damaged.
+     */
+    if (WIFSTOPPED (waitStatus)) {
+        Tcl_AppendResult (interp, "system command child stopped",
+                          (char *) NULL);
+        return TCL_ERROR;
+    }
+    return TCL_ERROR;  /* Should never reach here */
 }
 
 /*
@@ -359,11 +385,11 @@ Tcl_TimesCmd (clientData, interp, argc, argv)
 
     times(&tm);
 
-    sprintf(interp->result, "%ld %ld %ld %ld", 
-            tm.tms_utime  * MS_PER_TICK, 
-            tm.tms_stime  * MS_PER_TICK, 
-            tm.tms_cutime * MS_PER_TICK, 
-            tm.tms_cstime * MS_PER_TICK);
+    sprintf (interp->result, "%ld %ld %ld %ld", 
+             Tcl_TicksToMS (tm.tms_utime),
+             Tcl_TicksToMS (tm.tms_stime),
+             Tcl_TicksToMS (tm.tms_cutime),
+             Tcl_TicksToMS (tm.tms_cstime));
     return TCL_OK;
 }
 
