@@ -3,7 +3,7 @@
 #
 # Proc to execute code on every line of a file.
 #------------------------------------------------------------------------------
-# Copyright 1992-1993 Karl Lehenbauer and Mark Diekhans.
+# Copyright 1992-1995 Karl Lehenbauer and Mark Diekhans.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose and without fee is hereby granted, provided
@@ -12,18 +12,29 @@
 # software for any purpose.  It is provided "as is" without express or
 # implied warranty.
 #------------------------------------------------------------------------------
-# $Id: forfile.tcl,v 3.0 1993/11/19 07:00:24 markd Rel $
+# $Id: forfile.tcl,v 4.2 1995/01/01 19:49:56 markd Exp $
 #------------------------------------------------------------------------------
 #
 
 #@package: TclX-forfile for_file
 
-proc for_file {var filename code} {
+proc for_file {var filename cmd} {
     upvar $var line
     set fp [open $filename r]
+    set code 0
+    set result {}
     while {[gets $fp line] >= 0} {
-        uplevel $code
+        set code [catch {uplevel 1 $cmd} result]
+        if {$code != 0 && $code != 4} break
     }
     close $fp
-}
 
+    if {$code == 0 || $code == 3 || $code == 4} {
+        return $result
+    }
+    if {$code == 1} {
+        global errorCode errorInfo
+        return -code $code -errorcode $errorCode -errorinfo $errorInfo $result
+    }
+    return -code $code $result
+}
