@@ -4,7 +4,7 @@
 # C-shell style directory stack procs.
 #
 #------------------------------------------------------------------------------
-# Copyright 1992-1993 Karl Lehenbauer and Mark Diekhans.
+# Copyright 1992-1995 Karl Lehenbauer and Mark Diekhans.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose and without fee is hereby granted, provided
@@ -13,7 +13,7 @@
 # software for any purpose.  It is provided "as is" without express or
 # implied warranty.
 #------------------------------------------------------------------------------
-# $Id: pushd.tcl,v 3.0 1993/11/19 07:00:32 markd Rel $
+# $Id: pushd.tcl,v 5.0 1995/07/25 06:00:07 markd Rel $
 #------------------------------------------------------------------------------
 #
 
@@ -23,31 +23,42 @@ global TCLXENV(dirPushList)
 
 set TCLXENV(dirPushList) ""
 
-proc pushd {args} {
+proc pushd {{new ""}} {
     global TCLXENV
 
-    if {[llength $args] > 1} {
-        error "bad # args: pushd [dir_to_cd_to]"
-    }
-    set TCLXENV(dirPushList) [linsert $TCLXENV(dirPushList) 0 [pwd]]
-
-    if {[llength $args] != 0} {
-        cd [glob $args]
+    set current [pwd]
+    if {[clength $new] > 0} {
+        set dirs [glob -nocomplain $new]
+        set count [llength $dirs]
+        if {$count == 0} {
+            error "no such directory: $new"
+        } elseif {$count != 1} {
+            error "ambiguous directory: $new: [join $directories ", "]"
+        }
+        cd [lindex $dirs 0]
+        lvarpush TCLXENV(dirPushList) $current
+    } else {
+        if [lempty $TCLXENV(dirPushList)] {
+            error "directory stack empty"
+        }
+        cd [lindex $TCLXENV(dirPushList) 0]
+        lvarpop TCLXENV(dirPushList)
+        lvarpush TCLXENV(dirPushList) $current
     }
+    return [pwd]
 }
 
 proc popd {} {
     global TCLXENV
 
-    if [llength $TCLXENV(dirPushList)] {
-        cd [lvarpop TCLXENV(dirPushList)]
-        pwd
-    } else {
+    if [lempty $TCLXENV(dirPushList)] {
         error "directory stack empty"
     }
+    cd [lvarpop TCLXENV(dirPushList)]
+    return [pwd]
 }
 
 proc dirs {} { 
     global TCLXENV
-    echo [pwd] $TCLXENV(dirPushList)
+    return [concat [list [pwd]] $TCLXENV(dirPushList)]
 }
