@@ -4,7 +4,7 @@
 # Run the Tk demo at anytime after Extended Tcl is installed.
 # 
 #------------------------------------------------------------------------------
-# Copyright 1992-1993 Karl Lehenbauer and Mark Diekhans.
+# Copyright 1992-1995 Karl Lehenbauer and Mark Diekhans.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose and without fee is hereby granted, provided
@@ -13,7 +13,7 @@
 # software for any purpose.  It is provided "as is" without express or
 # implied warranty.
 #------------------------------------------------------------------------------
-# $Id: tkdemo.tcl,v 3.0 1993/11/19 07:01:10 markd Rel $
+# $Id: tkdemo.tcl,v 4.2 1995/01/01 19:50:10 markd Exp $
 #------------------------------------------------------------------------------
 #
 
@@ -26,7 +26,7 @@ proc tkdemo {} {
     }
     set demos [searchpath $auto_path demos]
     if {$demos == "" || ![file isdirectory $demos]} {
-        error "can't find Tk `demos' directory on the auto_path (auto_path):
+        error "can't find Tk `demos' directory on the auto_path (auto_path)"
     }
     uplevel #0 source $demos/widget
 }
