@@ -5,7 +5,7 @@
  * add to the master directory name.  This can be compiled to get either the
  * TclX or TkX version depending on if TK_GET_VERSION is defined.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1993 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1995 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -14,11 +14,12 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: getversion.c,v 3.0 1993/11/19 07:00:06 markd Rel $
+ * $Id: getversion.c,v 5.0 1995/07/25 05:59:33 markd Rel $
  *-----------------------------------------------------------------------------
  */
 
 #include "tclExtend.h"
+#include "tclXpatchl.h"
 
 #ifdef TK_GET_VERSION
 #include "tk.h"
@@ -30,9 +31,14 @@ main (argc, argv)
     char **argv;
 {
 #ifdef TK_GET_VERSION
-    printf ("%s%s\n", TK_VERSION, TCL_EXTD_VERSION_SUFFIX);
+    printf ("%s%s", TK_VERSION, TCL_EXTD_VERSION_SUFFIX);
 #else
-    printf ("%s%s\n", TCL_VERSION, TCL_EXTD_VERSION_SUFFIX);
+    printf ("%s%s", TCL_VERSION, TCL_EXTD_VERSION_SUFFIX);
 #endif
+#ifdef TCLX_PATCHLEVEL
+    if (TCLX_PATCHLEVEL != 0) 
+        printf ("-p%d", TCLX_PATCHLEVEL);
+#endif    
+    printf ("\n");
     exit (0);
 }
